﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SprawdzanieNumeryPesel.Model
{
    public enum Płeć { Nieznany, Kobieta, Mężczyzna };

    public class Pesel
    {
        public string NumerPesel { get; private set; }
        public bool Poprawny { get; private set; }
        public DateTime? DataUrodzenia { get; private set; }
        public int? NumerSeryjny { get; private set; }
        public Płeć Płeć { get; private set; }

        public static bool SprawdźNumerPesel(string pesel)
        {
            try
            {
                if (pesel.Length != 11) return false;
                int[] cyfry = new int[10];
                for (int i = 0; i < 10; ++i) cyfry[i] = int.Parse(pesel.Substring(i, 1));
                int cyfraKontrolna = int.Parse(pesel.Substring(10, 1));

                int sumaKontrolna = 9 * cyfry[0] + 7 * cyfry[1] + 3 * cyfry[2] + 1 * cyfry[3] + 9 * cyfry[4] + 7 * cyfry[5] + 3 * cyfry[6] + 1 * cyfry[7] + 9 * cyfry[8] + 7 * cyfry[9];
                sumaKontrolna = sumaKontrolna % 10;
                return sumaKontrolna == cyfraKontrolna;
            }
            catch
            {
                return false;
            }
        }

        private static DateTime? odkodujDatęUrodzenia(string pesel)
        {
            try
            {
                string srok = pesel.Substring(0, 2);
                string smiesiąc = pesel.Substring(2, 2);
                string sdzień = pesel.Substring(4, 2);
                int rok = int.Parse(srok);
                int miesiąc = int.Parse(smiesiąc);
                int dzień = int.Parse(sdzień);
                if (miesiąc >= 1 && miesiąc <= 12) rok += 1900;
                if (miesiąc >= 81 && miesiąc <= 92) { rok += 1800; miesiąc -= 80; }
                if (miesiąc >= 21 && miesiąc <= 32) { rok += 2000; miesiąc -= 20; }
                if (miesiąc >= 41 && miesiąc <= 52) { rok += 2100; miesiąc -= 40; }
                if (miesiąc >= 61 && miesiąc <= 72) { rok += 2200; miesiąc -= 60; }
                return new DateTime(rok, miesiąc, dzień);
            }
            catch
            {
                return null;
            }
        }

        private static int? odkodujNumerSeryjny(string pesel)
        {
            try
            {
                return int.Parse(pesel.Substring(6, 4));
            }
            catch
            {
                return null;
            }
        }

        private static Płeć odkodujPłeć(string pesel)
        {
            return (pesel[9] % 2 == 0) ? Płeć.Kobieta : Płeć.Mężczyzna;
        }

        //12345678901
        //44051401458

        public void ParsujNumerPesel(string pesel)
        {
            NumerPesel = pesel;
            bool poprawny = SprawdźNumerPesel(pesel);
            Poprawny = poprawny;
            if(poprawny)
            {
                DataUrodzenia = odkodujDatęUrodzenia(pesel);
                NumerSeryjny = odkodujNumerSeryjny(pesel);
                Płeć = odkodujPłeć(pesel);
            }
            else
            {
                DataUrodzenia = null;
                NumerSeryjny = null;
                Płeć = Płeć.Nieznany;
            }
        }
    }
}
